//+------------------------------------------------------------------+
//|                                                    Color RSI.mq4 |
//|                                                           mladen |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_separate_window
#property indicator_buffers   5
#property indicator_minimum   0
#property indicator_maximum 100
#property indicator_color1 DimGray
#property indicator_color2 LimeGreen
#property indicator_color3 LimeGreen
#property indicator_color4 Red
#property indicator_color5 Red
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2
#property indicator_width5 2


//---- input parameters
//
//
//
//

extern string    TimeFrame     =  "Current time frame";
extern int       KPeriod       = 21;
extern int       DPeriod       =  3;
extern int       Slowing       =  5;
extern int       MA_Method     =  3; // SMA 0, EMA 1, SMMA 2, LWMA 3
extern int       PriceType     =  0; // Low/High 0, Close/Close 1
extern int       Mode          =  MODE_SIGNAL;
extern int       overBought    =  50;
extern int       overSold      =  50;
extern bool      showArrows    = false;
extern bool      alertsOn      = true;
extern bool      alertsMessage = true;
extern bool      alertsSound   = false;
extern bool      alertsEmail   = true;
extern string    UniqueID      = "StocArrow";

//
//
//
//
//
//

double   StoBuffer[];
double   UpperUa[];
double   UpperUb[];
double   LowerDa[];
double   LowerDb[];
double   state[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,StoBuffer);
   SetIndexBuffer(1,UpperUa);
   SetIndexBuffer(2,UpperUb);
   SetIndexBuffer(3,LowerDa);
   SetIndexBuffer(4,LowerDb);
   SetIndexBuffer(5,state);
         
   //
   //
   //
   //
   //
       
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars";     if (returnBars)     { return(0); }
   calculateValue    = TimeFrame == "calculateValue"; if (calculateValue) { return(0); }
   timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //

         
      string shortName  = "STC ("+timeFrameToString(timeFrame)+","+KPeriod+","+Symbol()+" ";
            if (overBought < overSold) overBought = overSold;
            if (overBought < 100)      shortName  = shortName+","+overBought;
            if (overSold   >   0)      shortName  = shortName+","+overSold;
      IndicatorShortName(shortName+")");
   return(0);
}

//
//
//
//
//

int deinit()
{
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i); if (StringFind(name,UniqueID+":")==0) ObjectDelete(name);
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { StoBuffer[0] = MathMin(limit+1,Bars-1); return(0); }
         if (timeFrame!=Period())  limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            
      
   //
   //
   //
   //
   //
   
   if (state[limit]== 1) CleanPoint(limit,UpperUa,UpperUb);
   if (state[limit]==-1) CleanPoint(limit,LowerDa,LowerDb);
   for(int i=limit; i>=0; i--)
      {
            int y= iBarShift(NULL,timeFrame,Time[i]);
               StoBuffer[i] = iStochastic(NULL, timeFrame, KPeriod, DPeriod, Slowing,MA_Method, PriceType, Mode, y);
               UpperUa[i] = EMPTY_VALUE;
               UpperUb[i] = EMPTY_VALUE;
               LowerDa[i] = EMPTY_VALUE;
               LowerDb[i] = EMPTY_VALUE;
               state[i] = 0;
                  if (StoBuffer[i] > overBought) state[i] =  1;
                  if (StoBuffer[i] < overSold  ) state[i] = -1;
                  if (state[i] ==  1) PlotPoint(i,UpperUa,UpperUb,StoBuffer);
                  if (state[i] == -1) PlotPoint(i,LowerDa,LowerDb,StoBuffer);
                  if (showArrows  && state[i] != state[i+1])
                  {
                     if (state[i] ==  1) DrawArrow(i,"up");
                     if (state[i] == -1) DrawArrow(i,"down"); 
                  }                    
      }
      if (alertsOn  && state[0] != state[1])
      {
         if (state[0] ==  1) doAlert(overBought+" line crossed up");
         if (state[0] == -1) doAlert(overSold+" line crossed down");
      }                    
      return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void DrawArrow(int i,string type)
{
   string name = UniqueID+":"+Time[i];
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
      if (type=="up")
         {
            ObjectSet(name,OBJPROP_PRICE1,High[i]+3.0*iATR(NULL,0,20,i)/4.0);
            ObjectSet(name,OBJPROP_ARROWCODE,241);
            ObjectSet(name,OBJPROP_COLOR,Lime);
         }
      else
         {
            ObjectSet(name,OBJPROP_PRICE1,Low[i]-iATR(NULL,0,20,i)/2.0);
            ObjectSet(name,OBJPROP_ARROWCODE,242);
            ObjectSet(name,OBJPROP_COLOR,Red);
         }
}

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];
      
          //
          //
          //
          //
          //
            
          message =  StringConcatenate(Symbol(),Period()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," STOC ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsSound)   PlaySound("alert2.wav");
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),Period()," STOC crossing"),message);
      }        
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

